#!/usr/local/bin/ruby
# coding: utf-8




# 標準入力に対する一致反復率。周期性のある多表式暗号用


def print_usage
    print <<EOF
usage: friedman_test2.rb keylength
EOF
end



if ARGV.length < 1
    print_usage()
    exit
end

keylength = ARGV[0].to_i



text = ""
STDIN.each do |input_line|
    input_line.upcase!
    input_line.chop!
    text += input_line
end

# 記号や数字があるとアルファベットの出現頻度が変わって厄介なので
# A-Z以外の文字を捨てる
text.gsub!(/[^A-Z]/, "")


textbuf = text.split(//)



# テキストをkeylength*n+0文字目、keylength*n+1文字目,...に分割
# キーの文字数だけののテキストができる
splited_text = Array.new(keylength) { "" }
i = 0
while c = textbuf.shift
    splited_text[i] = splited_text[i] + c

    i += 1
    if i % keylength == 0
        i = 0
    end
end


# keylength個のテキストができるのでそれぞれ反復一致率を計算
total_index_coincidence = 0
splited_text.each do |t|
    freq = Hash.new
    ("A".."Z").each do |charactor|
        freq[charactor] = t.scan(charactor).size
    end

    index_coincidence = 0.0
    ("A".."Z").each do |charactor|
        index_coincidence += (freq[charactor].to_f / t.length) *
                             ((freq[charactor].to_f - 1) / (t.length - 1))
    end

    total_index_coincidence += index_coincidence
end

# 平均を取る
total_index_coincidence /= keylength




print "text length (alphabet only) = #{text.length}\n"
print "index of coincidence = #{total_index_coincidence}\n"
